@@ -34,7 +34,6 @@ except ImportError:
     
 import gi
 
-from gi.repository import GObject
 from gi.repository import GLib
 gi.require_version('Gtk', '3.0')
 from gi.repository import Gtk
@@ -52,15 +51,19 @@ import os
 import sys
 import time
 
+import animatedvalue
 import appconsts
 import audiomonitoring
 import audiowaveformrenderer
+import batchrendering
 import boxmove
+import callbackbridge
 import clipeffectseditor
 import clipmenuaction
 import compositeeditor
-import containerclip
+import compositormodes
 import containeractions
+import databridge
 import dialogs
 import dialogutils
 import dnd
@@ -80,6 +83,7 @@ import keyevents
 import keyframeeditor
 import keyframeeditcanvas
 import kftoolmode
+import medialinker
 import medialog
 import mediaplugin
 import mltenv
@@ -92,7 +96,6 @@ import movemodes
 import multitrimmode
 import persistance
 import positionbar
-import preferenceswindow
 import processutils
 import projectaction
 import projectdata
@@ -100,13 +103,13 @@ import projectdatavault
 import projectdatavaultgui
 import projectinfogui
 import propertyeditorbuilder
-import proxyediting
 import render
 import renderconsumer
 import rendergputest
 import respaths
 import resync
 import rotomask
+import scripttool
 import sequence
 import shortcuts
 import shortcutsquickeffects
@@ -130,6 +133,8 @@ import workflow
 _app = None
 _window = None
 
+_app_init_complete = False
+
 AUTOSAVE_DIR = appconsts.AUTOSAVE_DIR
 AUTOSAVE_FILE = "autosave/autosave"
 instance_autosave_id_str = None
@@ -140,18 +145,18 @@ disk_cache_timeout_id = -1
 loaded_autosave_file = None
 recovery_in_progress = False
 
-splash_screen = None
-splash_timeout_id = -1
 exit_timeout_id = -1
 window_resize_id = -1
 window_state_id = -1
 resize_timeout_id = -1
+monitor_resize_id = -1
 
 _log_file = None
 
 assoc_file_path = None
 assoc_timeout_id = None
 
+
 def main(root_path):
     """
     Called at application start.
@@ -226,7 +231,10 @@ class FlowbladeApplication(Gtk.Application):
         shortcuts.load_shortcut_files()
         shortcuts.load_shortcuts()
         shortcutsquickeffects.load_shortcuts()
-        
+
+        # We have translations in data structs and need some initing.
+        animatedvalue.init()
+
         # The test for len != 4 is to make sure that if we change the number of values below the prefs are reset to the correct list
         # So when we add or remove a value, make sure we also change the len test
         # Only use positive numbers.
@@ -248,7 +256,7 @@ class FlowbladeApplication(Gtk.Application):
             return
 
         # Apply custom themes.
-        gui.apply_theme(editorpersistance.prefs.theme)
+        gui.apply_theme()
 
         try:
             Gtk.Settings.get_default().set_property("gtk-application-prefer-dark-theme", True)
@@ -297,10 +305,6 @@ class FlowbladeApplication(Gtk.Application):
         # We need to test which GPU render options work after profiles are inited because
         # we do the test by doing test renders.
         rendergputest.test_gpu_rendering_options(render.update_encoding_selector)
-
-        # Splash screen
-        if editorpersistance.prefs.display_splash_screen == True: 
-            show_splash_screen()
             
         # Save assoc file path if found in arguments.
         global assoc_file_path
@@ -354,8 +358,17 @@ class FlowbladeApplication(Gtk.Application):
         # Inits widgets with current sequence data.
         init_sequence_gui()
 
-        # Launch player now that data and gui exist.
-        launch_player()
+        # Set SDL consumer version to be used.
+        #if editorstate.mlt_version_is_greater_correct("7.28.0") or editorstate.force_sdl2 == True \
+        #    or editorstate.app_running_from == editorstate.RUNNING_FROM_FLATPAK:
+        if editorstate.force_sdl2 == True:
+            mltplayer.set_sdl_consumer_version(mltplayer.SDL_2)
+        else:
+            mltplayer.set_sdl_consumer_version(mltplayer.SDL_1)
+
+        # Launch player if we're using SDL 1 consumer.
+        if mltplayer.get_sdl_consumer_version() == mltplayer.SDL_1:
+            launch_player()
 
         # Editor and modules need some more initializing.
         init_editor_state()
@@ -364,21 +377,17 @@ class FlowbladeApplication(Gtk.Application):
         global _window
         _window = gui.editor_window.window
         
-        # Tracks need to be re-centered if window is resized.
+        # Tracks need to be re-centered if window is resized and sdl2 consumer resized if
+        # monitor widget resized.
         # Connect listener for this now that the tline panel size allocation is sure to be available.
-        global window_resize_id, window_state_id
+        global window_resize_id, window_state_id, monitor_resize_id
         window_resize_id = gui.editor_window.window.connect("size-allocate", lambda w, e:updater.window_resized())
         window_state_id = gui.editor_window.window.connect("window-state-event", lambda w, e:updater.window_resized())
+        monitor_resize_id = gui.tline_display.connect("size-allocate", lambda w, e:tline_display_resized())
 
         # Get existing autosave files
         autosave_files = get_autosave_files()
 
-        # Show splash
-        if ((editorpersistance.prefs.display_splash_screen == True) and len(autosave_files) == 0):
-            global splash_timeout_id
-            splash_timeout_id = GLib.timeout_add(2600, destroy_splash_screen)
-            splash_screen.show_all()
-
         appconsts.SAVEFILE_VERSION = projectdata.SAVEFILE_VERSION # THIS IS A QUESTIONABLE IDEA TO SIMPLIFY IMPORTS, NOT DRY. WHEN DOING TOOLS THAT RUN IN ANOTHER PROCESSES AND SAVE PROJECTS, THIS LINE NEEDS TO BE THERE ALSO.
 
         # Every running instance has unique autosave file which is deleted at exit
@@ -408,12 +417,6 @@ class FlowbladeApplication(Gtk.Application):
                 print("Launch assoc file:", assoc_file_path)
                 global assoc_timeout_id
                 assoc_timeout_id = GLib.timeout_add(10, open_assoc_file)
-            
-        # SDL 2 consumer needs to created after Gtk.main() has run enough for window to be visible
-        #if editorstate.get_sdl_version() == editorstate.SDL_2: # needs more state consideration still
-        #    print "SDL2 timeout launch"
-        #    global sdl2_timeout_id
-        #    sdl2_timeout_id = GLib.timeout_add(1500, create_sdl_2_consumer)
         
         # In PositionNumericalEntries we are using Gtk.Entry objects in a way that works for us nicely, but is somehow "error" for Gtk, so we just kill this.
         Gtk.Settings.get_default().set_property("gtk-error-bell", False)
@@ -425,73 +428,85 @@ class FlowbladeApplication(Gtk.Application):
 
         # Connect to USB HID device (if enabled)
         start_usb_hid_input()
-    
+
+        # Used for imports refactoring.
+        for arg in sys.argv:
+            if arg == "--launchall":
+                GLib.timeout_add(1000, _launch_all)
+
+        global _app_init_complete
+        _app_init_complete = True
+
+        # Launch SDL 2 player now that data and gui exist if using that.
+        if mltplayer.get_sdl_consumer_version() == mltplayer.SDL_2:
+            GLib.idle_add(_create_sdl_2_consumer)
+
         self.add_window(_window)
+        if editorpersistance.prefs.global_layout != appconsts.SINGLE_WINDOW:
+            self.add_window(gui.editor_window.window2)
 
+# --------------------------------------- display
+def _create_sdl_2_consumer():
+    launch_player()
 
+def tline_display_resized():
+    editorstate.PLAYER().display_resized()
 
 # ----------------------------------- callback setting
 def monkeypatch_callbacks():
 
-    # We need to do this on app start-up or
-    # we'll get circular imports with projectaction->mltplayer->render->projectaction.
-    render.open_media_file_callback = projectaction.open_rendered_file
-    jobs.open_media_file_callback = projectaction.open_rendered_file
+    callbackbridge.app_open_project = open_project
+    callbackbridge.app_new_project = new_project
+    callbackbridge.app_stop_autosave = stop_autosave
+    callbackbridge.app_start_autosave = start_autosave
+    callbackbridge.app_change_current_sequence = change_current_sequence
+    callbackbridge.app_shutdown = shutdown
+    callbackbridge.boxmove_get_selection_data = boxmove.get_selection_data
+    callbackbridge.clipeffectseditor_refresh_clip = clipeffectseditor.refresh_clip
+    callbackbridge.clipeffectseditor_update_kfeditors_sliders = clipeffectseditor.update_kfeditors_sliders
+    callbackbridge.clipeffectseditor_clip_is_being_edited = clipeffectseditor.clip_is_being_edited
+    callbackbridge.clipmenuaction_display_clip_menu = clipmenuaction.display_clip_menu
+    callbackbridge.clipmenuaction_compositor_menu_item_activated = clipmenuaction.compositor_menu_item_activated
+    callbackbridge.clipmenuaction_get_popover_clip_data = clipmenuaction.get_popover_clip_data
+    callbackbridge.clipmenuaction_set_compositor_data = clipmenuaction.set_compositor_data
+    callbackbridge.editevent_tline_range_item_drop = editevent.tline_range_item_drop
+    callbackbridge.compositeeditor_get_compositor = compositeeditor.get_compositor
+    callbackbridge.compositormodes_get_snapped_x = compositormodes.get_snapped_x
+    callbackbridge.compositormodes_get_pointer_context = compositormodes.get_pointer_context
+    callbackbridge.dialogs_show_cyclic_error = dialogs.show_cyclic_error 
+    callbackbridge.editevent_do_multiple_clip_insert = editevent.do_multiple_clip_insert
+    callbackbridge.medialog_clips_drop = medialog.clips_drop
+    callbackbridge.mediaplugin_get_clip = mediaplugin.get_clip
+    callbackbridge.mediaplugin_set_plugin_to_be_edited = mediaplugin.set_plugin_to_be_edited
+    callbackbridge.modesetting_set_default_edit_mode = modesetting.set_default_edit_mode
+    callbackbridge.modesetting_kftool_mode_from_kf_editor = modesetting.kftool_mode_from_kf_editor
+    callbackbridge.movemodes_select_clip = movemodes.select_clip
+    callbackbridge.movemodes_select_from_box_selection = movemodes.select_from_box_selection
+    callbackbridge.projectaction_open_rendered_file = projectaction.open_rendered_file
+    callbackbridge.projectaction_open_file_names = projectaction.open_file_names
+    callbackbridge.rotomask_show_rotomask = rotomask.show_rotomask
+    callbackbridge.targetactions_get_handler_by_name = targetactions.get_handler_by_name
+    callbackbridge.targetactions_move_player_position = targetactions.move_player_position
+    callbackbridge.targetactions_variable_speed_playback = targetactions.variable_speed_playback
+    callbackbridge.trimmodes_set_no_edit_trim_mode = trimmodes.set_no_edit_trim_mode
+    callbackbridge.updater_display_sequence_in_monitor = updater.display_sequence_in_monitor
+    callbackbridge.updater_set_and_display_monitor_media_file = updater.set_and_display_monitor_media_file
+    callbackbridge.updater_repaint_tline = updater.repaint_tline
+    
+    databridge.app_get_save_time_msg = get_save_time_msg
+    databridge.mltprofiles_get_profile_name_for_index = mltprofiles.get_profile_name_for_index
+    databridge.tlinewidgets_get_frame = tlinewidgets.get_frame
+    databridge.tlinewidgets_get_frame_x = tlinewidgets._get_frame_x
+    databridge.tlinewidgets_get_track = tlinewidgets.get_track
+    databridge.trimmodes_submode_is_keyb_edit_on = trimmodes.submode_is_keyb_edit_on
+    databridge.snapping_get_snapping_on = snapping.get_snapping_on
+    databridge.usbhid_get_usb_hid_device_config_metadata_list = usbhid.get_usb_hid_device_config_metadata_list
 
-    # Set callback for undo/redo ops, batcherrender app does not need this.
+    # Set callbacks for undo/redo ops.
     undo.set_post_undo_redo_callback(modesetting.set_post_undo_redo_edit_mode)
     undo.repaint_tline = updater.repaint_tline
 
-    # Drag'n'drop callbacks
-    dnd.display_monitor_media_file = updater.set_and_display_monitor_media_file
-    dnd.range_log_items_tline_drop = editevent.tline_range_item_drop
-    dnd.range_log_items_log_drop = medialog.clips_drop
-    dnd.open_dropped_files = projectaction.open_file_names
 
-    # Media log 
-    medialog.do_multiple_clip_insert_func = editevent.do_multiple_clip_insert
-
-    editevent.display_clip_menu_pop_up = clipmenuaction.display_clip_menu
-    editevent.compositor_menu_item_activated = clipmenuaction.compositor_menu_item_activated
-    editevent.set_compositor_data = clipmenuaction.set_compositor_data
-    
-    # Posionbar in gmic.py does not need trimmodes.py dependency and is avoided.
-    positionbar.trimmodes_set_no_edit_trim_mode = trimmodes.set_no_edit_trim_mode
-
-    # Snapping is done in a separate module but needs some tlinewidgets state info
-    snapping._get_frame_for_x_func = tlinewidgets.get_frame
-    snapping._get_x_for_frame_func = tlinewidgets._get_frame_x
-
-    # Callback to reinit to change slider <-> kf editor.
-    propertyeditorbuilder.re_init_editors_for_slider_type_change_func = clipeffectseditor.refresh_clip
-
-    propertyeditorbuilder.show_rotomask_func = rotomask.show_rotomask
-    
-    multitrimmode.set_default_mode_func = modesetting.set_default_edit_mode
-    
-    keyframeeditor._get_current_edited_compositor = compositeeditor.get_compositor
-
-    guicomponents.select_clip_func = movemodes.select_clip
-    guipopoverclip.select_clip_func = movemodes.select_clip
-    
-    containeractions.set_plugin_to_be_edited_func = mediaplugin.set_plugin_to_be_edited
-    containeractions.get_edited_plugin_clip = mediaplugin.get_clip
-    
-    boxmove.set_move_selection_from_box_selection_func = movemodes.select_from_box_selection
-
-    toolsintegration.get_popover_clip_data_func = clipmenuaction.get_popover_clip_data
-
-    usbhiddrivers.targetactions_get_handler_by_name_func = targetactions.get_handler_by_name
-    usbhiddrivers.targetactions_move_player_position_func = targetactions.move_player_position
-    usbhiddrivers.targetactions_variable_speed_playback_func = targetactions.variable_speed_playback
-
-    # These provide clues for possible further refactoring.
-
-# ---------------------------------- SDL2 consumer
-#def create_sdl_2_consumer():
-#    GObject.source_remove(disk_cache_timeout_id)
-#    print "Creating SDL2 consumer..."
-#    editorstate.PLAYER().create_sdl2_video_consumer()
 
 # ---------------------------------- program, sequence and project init
 def get_assoc_file_path():
@@ -543,6 +558,8 @@ def create_gui():
     editorlayout.set_positions_frames_visibility()
         
     # All widgets are now realized and references captured so can find out theme colors.
+    # TODO: Delete this code, we are not detecting anything anymore and theme colors not
+    # variable anymore.
     gui.set_theme_colors()
     tlinewidgets.set_dark_bg_color()
     gui.pos_bar.set_dark_bg_color()
@@ -779,6 +796,10 @@ def change_current_sequence(index):
     updater.init_tline_view()
 
 def display_current_sequence():
+    # On launch we need to set up SDL player differently.
+    if _app_init_complete == False:
+        return
+
     # Get shorter alias.
     player = editorstate.player
 
@@ -878,27 +899,6 @@ def do_autosave():
     persistance.save_project(editorstate.PROJECT(), autosave_file)
     return True
 
-# ------------------------------------------------- splash screen
-def show_splash_screen():
-    global splash_screen
-    splash_screen = Gtk.Window(Gtk.WindowType.TOPLEVEL)
-    splash_screen.set_border_width(0)
-    splash_screen.set_decorated(False)
-    splash_screen.set_position(Gtk.WindowPosition.CENTER)
-    img = Gtk.Image.new_from_file(respaths.IMAGE_PATH + "flowblade_splash_black_small.png")
-
-    splash_screen.add(img)
-    splash_screen.set_size_request(598, 258) # Smaller then img.
-
-    splash_screen.set_resizable(False)
-
-    while(GLib.MainContext.default ().pending()):
-        GLib.MainContext.default().iteration(False) # GLib.MainContext to replace this
-
-def destroy_splash_screen():
-    splash_screen.destroy()
-    GLib.source_remove(splash_timeout_id)
-
 # ------------------------------------------------------- disk cache size check
 def check_disk_cache_size():
     GLib.source_remove(disk_cache_timeout_id)
@@ -921,6 +921,15 @@ def xdg_data_dir_changed_dialog():
         dialogutils.warning_message(primary_txt, secondary_txt, gui.editor_window.window)
     return False
 
+# -------------------------------------------------------- launch tests for imports refactoring testing
+def _launch_all():
+    gmic.launch_gmic()
+    scripttool.launch_scripttool()
+    medialinker.display_linker()
+    batchrendering.launch_batch_rendering()
+
+    return False
+
 # ------------------------------------------------------- small and multiple screens
 # We need a bit more stuff because having multiple monitors can mix up setting draw parameters.
 def _set_screen_size_data():
@@ -966,15 +975,24 @@ def _set_draw_params():
     if editorstate.screen_size_large_height() == True:
         keyframeeditcanvas.GEOMETRY_EDITOR_HEIGHT = 300
 
-    if editorpersistance.prefs.double_track_hights == True:
+    if editorpersistance.prefs.tracks_scale == appconsts.TRACKS_SCALE_DEFAULT:
+        return
+    elif editorpersistance.prefs.tracks_scale == appconsts.TRACKS_SCALE_DOUBLE:
         appconsts.TRACK_HEIGHT_HIGH = 150
         appconsts.TRACK_HEIGHT_NORMAL = 100 # track height in canvas and column
         appconsts.TRACK_HEIGHT_SMALL = 50 # track height in canvas and column
         appconsts.TLINE_HEIGHT = 520
-        sequence.TRACK_HEIGHT_NORMAL = appconsts.TRACK_HEIGHT_NORMAL # track height in canvas and column
-        sequence.TRACK_HEIGHT_SMALL = appconsts.TRACK_HEIGHT_SMALL # track height in canvas and column
-        sequence.TRACK_HEIGHT_HIGH = appconsts.TRACK_HEIGHT_HIGH
-        tlinewidgets.set_tracks_double_height_consts()
+    else:
+        # TRACKS_SCALE_ONE_AND_HALF
+        appconsts.TRACK_HEIGHT_HIGH = 113
+        appconsts.TRACK_HEIGHT_NORMAL = 75 # track height in canvas and column
+        appconsts.TRACK_HEIGHT_SMALL = 37 # track height in canvas and column
+        appconsts.TLINE_HEIGHT = 390
+                
+    sequence.TRACK_HEIGHT_NORMAL = appconsts.TRACK_HEIGHT_NORMAL # track height in canvas and column
+    sequence.TRACK_HEIGHT_SMALL = appconsts.TRACK_HEIGHT_SMALL # track height in canvas and column
+    sequence.TRACK_HEIGHT_HIGH = appconsts.TRACK_HEIGHT_HIGH
+    tlinewidgets.set_tracks_height_consts()
 
 def _too_small_screen_exit():
     global exit_timeout_id
@@ -1150,13 +1168,13 @@ def _app_destroy():
     stop_usb_hid_input()
 
     # Now that autosave file is deleted, the data folder contents for unsaved
-    # projects are unreachable and can be destoyed.
+    # projects are unreachable and can be destroyed.
     projectdatavault.delete_unsaved_data_folders()
 
     # Calling _app.quit() will block if Gio.Application.hold() has been called,
     # and the documentation helpfully says "(Note that you may have called 
     # Gio.Application.hold() indirectly, for example through gtk_application_add_window().)"
-    # I'm not calling add_window() on render processess but somehow those call Gio.Application.hold()
+    # I'm not calling add_window() on render processes but somehow those call Gio.Application.hold()
     # but I don't know where. 
     #
     # However, those processes complete and exit cleanly and the only problem 
