@@ -2,7 +2,7 @@
     Flowblade Movie Editor is a nonlinear video editor.
     Copyright 2012 Janne Liljeblad.
 
-    This file is part of Flowblade Movie Editor <http://code.google.com/p/flowblade>.
+    This file is part of Flowblade Movie Editor <https://github.com/jliljebl/flowblade/>.
 
     Flowblade Movie Editor is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,14 +19,17 @@
 """
 
 """
-Module contains GUI widgets used edit geometries on canvas with a mouse.
+Module contains GUI widgets used to edit geometries on canvas with a mouse.
 """
+import traceback
+
 
 import copy
 import math
 
 from gi.repository import Gdk
 
+import animatedvalue
 import appconsts
 import cairoarea
 import utils
@@ -214,7 +217,7 @@ class EditRect:
         # handles
         for id_int, pos in self.edit_points.items():
             x, y = pos
-            cr.rectangle(x - 2, y - 2, 4, 4)
+            cr.rectangle(x - EP_HALF, y - EP_HALF, EP_HALF * 2.0,  EP_HALF * 2.0)
             cr.fill()
         
         if self.projection_point != None:
@@ -230,7 +233,14 @@ def _geom_kf_sort(kf):
     """
     frame, shape, opacity, type = kf
     return frame 
-        
+
+def _gradient_kf_sort(kf):
+    """
+    Function is used to sort keyframes by frame number.
+    """
+    frame, values, type = kf
+    return frame 
+    
 
 class AbstractEditCanvas:
     """
@@ -276,6 +286,8 @@ class AbstractEditCanvas:
         self.source_height = -1 # unscaled source image height, set later
         
         self.coords = None # Calculated later when we have allocation available
+
+        self.active = True # used to disable if needed.
         
     def init_editor(self, source_width, source_height, y_fract):
         self.source_width = source_width 
@@ -358,12 +370,24 @@ class AbstractEditCanvas:
         self.keyframes.append((frame, copy.deepcopy(p_shape), copy.deepcopy(p_opacity),  p_type))
         
         self.keyframes.sort(key=_geom_kf_sort)
+
+    def add_keyframe_with_shape_opacity_and_type(self, frame, shape, opacity, kf_type):
+        if self._frame_has_keyframe(frame) == True:
+            kf_index = self._get_frame_keyframe_index(frame)
+            self.keyframes.pop(kf_index)
+            self._update_shape()
+
+        # Add with values, for now we always set opacity to max.
+        self.keyframes.append((frame, shape, opacity,  kf_type))
+        
+        self.keyframes.sort(key=_geom_kf_sort)
         
     def delete_active_keyframe(self, keyframe_index):
         if keyframe_index == 0:
             # keyframe frame 0 cannot be removed
             return
         self.keyframes.pop(keyframe_index)
+        self._update_shape()
 
     def _frame_has_keyframe(self, frame):
         for i in range(0, len(self.keyframes)):
@@ -374,19 +398,42 @@ class AbstractEditCanvas:
 
         return False
 
+    def _get_frame_keyframe_index(self, frame):
+        for i in range(0, len(self.keyframes)):
+            kf = self.keyframes[i]
+            kf_frame, rect, opacity, kf_type = kf
+            if frame == kf_frame:
+                return i
+
+        return None
+        
     def set_keyframes(self, keyframes_str, out_to_in_func):
         self.keyframes = self.keyframe_parser(keyframes_str, out_to_in_func)
 
     def set_keyframe_frame(self, active_kf_index, frame):
-        old_frame, shape, opacity, kf_type = self.keyframes[active_kf_index]
-        self.keyframes.pop(active_kf_index)
-        self.keyframes.insert(active_kf_index, (frame, shape, opacity, kf_type))    
-
+        try:
+            # 4 values in kf tuple
+            old_frame, shape, opacity, kf_type = self.keyframes[active_kf_index]
+            self.keyframes.pop(active_kf_index)
+            self.keyframes.insert(active_kf_index, (frame, shape, opacity, kf_type))    
+        except:
+            # 3 values in kf tuple
+            old_frame, value, kf_type = self.keyframes[active_kf_index]
+            self.keyframes.pop(active_kf_index)
+            self.keyframes.insert(active_kf_index, (frame, value, kf_type))    
+            
     def set_active_kf_type(self, active_kf_index, kf_type):
-        old_frame, shape, opacity, old_kf_type = self.keyframes[active_kf_index]
-        self.keyframes.pop(active_kf_index)
-        self.keyframes.insert(active_kf_index, (old_frame, shape, opacity, kf_type))    
-        
+        try:
+            # 4 values in kf tuple
+            old_frame, shape, opacity, old_kf_type = self.keyframes[active_kf_index]
+            self.keyframes.pop(active_kf_index)
+            self.keyframes.insert(active_kf_index, (old_frame, shape, opacity, kf_type))    
+        except:
+            # 3 values in kf tuple
+            old_frame, value, old_kf_type = self.keyframes[active_kf_index]
+            self.keyframes.pop(active_kf_index)
+            self.keyframes.insert(active_kf_index, (old_frame, value, kf_type))
+
     def get_keyframe(self, kf_index):
         return self.keyframes[kf_index]
 
@@ -421,6 +468,9 @@ class AbstractEditCanvas:
         """
         Mouse button callback
         """
+        if self.active == False:
+            return
+
         self.current_mouse_hit = self._check_shape_hit(event.x, event.y)
         if self.current_mouse_hit == NO_HIT:
             return
@@ -443,6 +493,9 @@ class AbstractEditCanvas:
         """
         Mouse move callback
         """
+        if self.active == False:
+            return
+            
         if self.current_mouse_hit == NO_HIT:
             return
         
@@ -467,6 +520,9 @@ class AbstractEditCanvas:
         print("_shape__motion_notify_event not impl")
 
     def _release_event(self, event):
+        if self.active == False:
+            return
+
         global _shift_down 
         _shift_down = None
         
@@ -505,7 +561,7 @@ class AbstractEditCanvas:
     def _draw(self, event, cr, allocation):
         """
         Callback for repaint from CairoDrawableArea.
-        We get cairo contect and allocation.
+        We get cairo context and allocation.
         """
         if self.coords == None:
             self._create_coords()
@@ -540,6 +596,11 @@ class AbstractEditCanvas:
         
         self._draw_edit_shape(cr, allocation)
 
+        if self.active == False:
+            cr.set_source_rgba(0.75, 0.75, 0.77, 0.65)
+            cr.rectangle(0, 0, w, h)
+            cr.fill()
+    
     def _draw_edge(self, cr, rect):
         cr.set_line_width(1.0)
         cr.set_source_rgb(0, 0, 0)
@@ -549,6 +610,11 @@ class AbstractEditCanvas:
     def _draw_edit_shape(self, cr, allocation):
         print("_draw_edit_shape not impl.")
         
+    def print_keyframes(self):
+        print("Keyframes:")
+        for i in range(0, len(self.keyframes)):
+            print(self.keyframes[i])
+
 
 class BoxEditCanvas(AbstractEditCanvas):
     """
@@ -601,6 +667,28 @@ class BoxEditCanvas(AbstractEditCanvas):
         self.keyframes.pop(active_kf_index)
         self.keyframes.insert(active_kf_index, (frame, rect, opacity, kf_type))
 
+    def clone_value_from_next(self, active_kf_index):
+        frame, rect, opacity, kf_type = self.keyframes.pop(active_kf_index)
+
+        try:
+            frame_n, rect_n, opacity_n, kf_type_n = self.keyframes[active_kf_index]
+        except:
+            # No next keyframe
+            return
+    
+        self.keyframes.insert(active_kf_index, (frame, rect_n, opacity_n, kf_type))
+        self.parent_editor.update_slider_value_display(self.current_clip_frame)
+
+    def clone_value_from_prev(self, active_kf_index):
+        if active_kf_index == 0:
+            return
+            
+        frame, rect, opacity, kf_type = self.keyframes.pop(active_kf_index)
+        frame_n, rect_n, opacity_n, kf_type_n = self.keyframes[active_kf_index - 1]
+    
+        self.keyframes.insert(active_kf_index, (frame, rect_n, opacity_n, kf_type))
+        self.parent_editor.update_slider_value_display(self.current_clip_frame)
+        
     def _clip_frame_changed(self):
         if self.source_edit_rect != None:
             self._update_source_rect()
@@ -627,12 +715,8 @@ class BoxEditCanvas(AbstractEditCanvas):
                     if kf_type == appconsts.KEYFRAME_DISCRETE:
                         self.set_geom(*rect)
                         return
-                    elif kf_type == appconsts.KEYFRAME_SMOOTH:
-                        frame_rect = self._get_interpolated_rect_smooth(time_fract, i, self.keyframes)
-                        self.source_edit_rect.set_geom(*self._get_screen_to_panel_rect(frame_rect))
-                        return
-                    else: # LINEAR
-                        frame_rect = self._get_interpolated_rect(rect, rect_n, time_fract)
+                    else: # interpolated values
+                        frame_rect = self._get_interpolated_rect(time_fract, i)
                         self.source_edit_rect.set_geom(*self._get_screen_to_panel_rect(frame_rect))
                         return
             except: # past last frame, use its value
@@ -640,49 +724,29 @@ class BoxEditCanvas(AbstractEditCanvas):
                 return
                 
         print("reached end of _update_source_rect, this should be unreachable")
-        
-    def _get_interpolated_rect(self, rect_1, rect_2, fract):
-        x1, y1, w1, h1 = rect_1
-        x2, y2, w2, h2 = rect_2
-        x = x1 + (x2 - x1) * fract
-        y = y1 + (y2 - y1) * fract
-        w = w1 + (w2 - w1) * fract
-        h = h1 + (h2 - h1) * fract
-        return (x, y, w, h)
-
-    def _get_interpolated_rect_smooth(self, fract, i, keyframes):
-        prev = i
-        if i == 0:
-            prev_prev = 0
-        else:
-            prev_prev = i - 1
-        
-        next = i + 1
-        if next >= len(keyframes):
-            next = len(keyframes) - 1
-        
-        next_next = next + 1
-        if next_next >= len(keyframes):
-            next_next = len(keyframes) - 1
-
-        frame, rect, opacity, kf_type = keyframes[prev_prev]
-        x0, y0, w0, h0 = rect
-
-        frame, rect, opacity, kf_type = keyframes[prev]
-        x1, y1, w1, h1 = rect
-
-        frame, rect, opacity, kf_type = keyframes[next]
-        x2, y2, w2, h2 = rect
-
-        frame, rect, opacity, kf_type = keyframes[next_next]
-        x3, y3, w3, h3 = rect
-
-        x = self.catmull_rom_interpolate(x0, x1, x2, x3, fract)
-        y = self.catmull_rom_interpolate(y0, y1, y2, y3, fract)
-        w = self.catmull_rom_interpolate(w0, w1, w2, w3, fract)
-        h = self.catmull_rom_interpolate(h0, h1, h2, h3, fract)
+    
+    def _get_interpolated_rect(self, fract, i):
+        anim_value_x = self._create_anim_value(0)
+        x = anim_value_x.get_interpolated_value_internal_kf_type(i, fract)
+        anim_value_y = self._create_anim_value(1)
+        y = anim_value_y.get_interpolated_value_internal_kf_type(i, fract)
+        anim_value_w = self._create_anim_value(2)
+        w = anim_value_w.get_interpolated_value_internal_kf_type(i, fract)
+        anim_value_h = self._create_anim_value(3)
+        h = anim_value_h.get_interpolated_value_internal_kf_type(i, fract)
 
         return (x, y, w, h)
+
+    def _create_anim_value(self, value_index):
+        
+        value_keyframes = []
+    
+        for kf in self.keyframes:
+            frame, rect, opacity, kf_type = kf
+            value = rect[value_index] # x, y, w, h
+            value_keyframes.append((frame, value, kf_type))
+
+        return animatedvalue.AnimatedValue(value_keyframes)
         
     def _get_screen_to_panel_rect(self, rect):
         x, y, w, h = rect
@@ -762,9 +826,251 @@ class BoxEditCanvas(AbstractEditCanvas):
         
         self.source_edit_rect.h = self.source_edit_rect.h * (self.source_edit_rect.w / old_w)
 
-    def print_keyframes(self):
+
+
+class GradientEditCanvas(AbstractEditCanvas):
+    """
+    GUI component for editing position and scale values of keyframes 
+    of source image in compositors. 
+    
+    Component is used as a part of e.g GeometryEditor, which handles
+    also keyframe creation and deletion and opacity, and
+    writing out the keyframes with combined information.
+
+    Required parent_editor callback interface:
+        mouse_scroll_up()
+        mouse_scroll_down()
+        geometry_edit_started()
+        update_request_from_geom_editor()
+        queue_draw()
+        geometry_edit_finished()
+    """
+    def __init__(self, editable_property, parent_editor):
+        AbstractEditCanvas.__init__(self, editable_property, parent_editor)
+        self.edit_points = []
+            
+    def create_edit_points_and_values(self):
+        # creates untransformed edit shape to init array, values will be overridden shortly
+        self.edit_points.append((self.source_width / 2, self.source_height / 2 + self.source_height / 4))  # center
+        self.edit_points.append((self.source_width / 2, self.source_height / 2 - self.source_height / 4)) # center
+
+        self.untrans_points = copy.deepcopy(self.edit_points)
+
+    def _frame_has_keyframe(self, frame):
         for i in range(0, len(self.keyframes)):
-            print(self.keyframes[i])
+            kf = self.keyframes[i]
+            kf_frame, values, kf_type = kf
+            if frame == kf_frame:
+                return True
+
+        return False
+
+    def _get_frame_keyframe_index(self, frame):
+        for i in range(0, len(self.keyframes)):
+            kf = self.keyframes[i]
+            kf_frame, value, kf_type = kf
+            if frame == kf_frame:
+                return i
+
+        return None
+
+    def add_keyframe(self, frame):
+        if self._frame_has_keyframe(frame) == True:
+            return
+
+        # Get previous keyframe
+        prev_kf = None
+        for i in range(0, len(self.keyframes)):
+            p_frame, p_values, p_type = self.keyframes[i]
+            if p_frame < frame:
+                prev_kf = self.keyframes[i]                
+        if prev_kf == None:
+            prev_kf = self.keyframes[len(self.keyframes) - 1]
+        
+        # Add with values of previous
+        p_frame, p_values, p_type  = prev_kf
+        self.keyframes.append((frame, copy.deepcopy(p_values),  p_type))
+        
+        self.keyframes.sort(key=_gradient_kf_sort)
+
+    def clone_value_from_next(self, active_kf_index):
+        frame, values, kf_type = self.keyframes.pop(active_kf_index)
+
+        try:
+            frame_n, values_n, kf_type_n = self.keyframes[active_kf_index]
+        except:
+            # No next keyframe
+            return
+    
+        self.keyframes.insert(active_kf_index, (frame, values_n, kf_type))
+        self.parent_editor.update_slider_value_display(self.current_clip_frame)
+
+    def clone_value_from_prev(self, active_kf_index):
+        if active_kf_index == 0:
+            return
+            
+        frame, values, kf_type = self.keyframes.pop(active_kf_index)
+        frame_p, values_p, kf_type_p = self.keyframes[active_kf_index - 1]
+    
+        self.keyframes.insert(active_kf_index, (frame, values_p, kf_type))
+        self.parent_editor.update_slider_value_display(self.current_clip_frame)
+        
+    def _clip_frame_changed(self):
+        self._update_shape()
+
+    def _update_shape(self):
+        for i in range(0, len(self.keyframes)):
+            frame, values, kf_type = self.keyframes[i]
+
+            if frame == self.current_clip_frame:
+                # current_clip_frame is on keyframe. 
+                self.set_geom(values)
+                return
+                
+            # Check if frame between these keyframes and interpolate and update shape if so.
+            try:
+                frame_n, value_n, kf_type_n = self.keyframes[i + 1]
+                if ((frame < self.current_clip_frame)
+                    and (self.current_clip_frame < frame_n)):
+
+                    time_fract = float((self.current_clip_frame - frame)) / \
+                                 float((frame_n - frame))
+                    if kf_type == appconsts.KEYFRAME_DISCRETE:
+                        self.set_geom(values)
+                        return
+                    else:
+                        interpolated_values = self._get_interpolated_values(time_fract, i, kf_type)
+                        self.set_geom(interpolated_values)
+                        return
+            
+            except Exception as e:
+                # Getting next kf info crashes because past last frame, use its value.
+                self.set_geom(values)
+                return
+
+    def _get_interpolated_values(self, fract, kf_index, kf_type):
+        
+        anim_value_start_x = self._create_anim_value(0)
+        start_x_val = anim_value_start_x.get_interpolated_value(kf_index, fract, kf_type)
+        
+        anim_value_start_y = self._create_anim_value(1)
+        start_y_val = anim_value_start_y.get_interpolated_value(kf_index, fract, kf_type)
+        
+        anim_value_end_x = self._create_anim_value(2)
+        end_x_val = anim_value_end_x.get_interpolated_value(kf_index, fract, kf_type)
+
+        anim_value_end_y = self._create_anim_value(3)
+        end_y_val = anim_value_end_y.get_interpolated_value(kf_index, fract, kf_type)
+
+        return (start_x_val, start_y_val, end_x_val, end_y_val)
+
+    def _create_anim_value(self, value_index):
+        
+        value_keyframes = []
+    
+        for kf in self.keyframes:
+            frame, values, kf_type = kf
+            value = values[value_index] # start_x, start_y, end_x, end_y = values
+            value_keyframes.append((frame, value, kf_type))
+
+        return animatedvalue.AnimatedValue(value_keyframes)
+
+    def set_geom(self,values):
+        # Set edit point to position defined by keyframe values.
+        # keyframe values 0 - 1, edipoint asre 0 - screen width/height
+        x1, y1, x2, y2 = values
+        p1 = (x1, y1)
+        p2 = (x2, y2)
+        self.edit_points = [p1, p2]
+
+    def _draw_edit_shape(self, cr, allocation):
+        x1, y1 = self.get_panel_point(*self.edit_points[0])
+        x2, y2 = self.get_panel_point(*self.edit_points[1])
+        self._draw_edit_point(cr, x1, y1)
+        self._draw_edit_point(cr, x2, y2)
+
+        cr.move_to(x1, y1)
+        cr.line_to(x2, y2)
+        cr.stroke()
+
+    def _draw_edit_point(self, cr, px, py):
+        CROSS_HALF = 6
+        cr.move_to(px - CROSS_HALF, py)
+        cr.line_to(px + CROSS_HALF, py)
+        cr.stroke()
+
+        cr.move_to(px, py - CROSS_HALF,)
+        cr.line_to(px, py + CROSS_HALF)
+        cr.stroke()
+        
+    # ----------------------------------------- mouse press event
+    def _check_shape_hit(self, x, y):
+        edit_panel_points = []
+        for ep in self.edit_points:
+            edit_panel_points.append(self.get_panel_point(*ep))
+        
+        for i in range(0, 2):
+            if self._check_point_hit((x, y), edit_panel_points[i], 10):
+                return i #indexes correspond to edit_point_handle indexes
+        
+        return NO_HIT
+    
+    def _check_point_hit(self, p, ep, TARGET_HALF):
+        x, y = p
+        ex, ey = ep
+        if (x >= ex - TARGET_HALF and x <= ex + TARGET_HALF and y >= ey - TARGET_HALF and y <= ey + TARGET_HALF):
+            return True
+
+        return False
+
+    def _shape_press_event(self):
+        self.start_edit_points = copy.deepcopy(self.edit_points)
+
+    def _shape__motion_notify_event(self, delta_x, delta_y, CTRL_DOWN):
+        self._save_edited_point(delta_x, delta_y, CTRL_DOWN)
+        
+    def _shape_release_event(self, delta_x, delta_y, CTRL_DOWN):
+        self._save_edited_point(delta_x, delta_y, CTRL_DOWN)
+        
+    def _save_edited_point(self, delta_x, delta_y, CTRL_DOWN):
+        # Convert unedited point to panel coords, add mouse delta, 
+        # convert back to screen coords, update edited point value.
+        target_point = self.start_edit_points[self.current_mouse_hit] # current_mouse_hit was set to be index of pressed edit point
+        px, py = self.get_panel_point(*target_point)
+        new_px = px + delta_x
+        new_py = py + delta_y 
+        ep_x = self.get_screen_x(new_px)
+        ep_y = self.get_screen_y(new_py)
+        self.edit_points.pop(self.current_mouse_hit)
+        self.edit_points.insert(self.current_mouse_hit, (ep_x, ep_y))
+        
+    def set_keyframe_to_edit_shape(self, kf_index, value_shape=None):
+        if value_shape == None:
+            current_values = self._get_current_screen_shape()
+        
+        frame, values, kf_type = self.keyframes[kf_index]
+        self.keyframes.pop(kf_index)
+        
+        new_kf = (frame, current_values, kf_type)
+        self.keyframes.append(new_kf)
+        self.keyframes.sort(key=_gradient_kf_sort)
+
+        self._update_shape()
+
+    def _get_current_screen_shape(self):
+        x1, y1 = self.edit_points[0]
+        x2, y2 = self.edit_points[1]
+        return (x1, y1, x2, y2)
+
+    def handle_arrow_edit(self, keyval, delta):
+        print("handle_arrow_edit")
+
+    def handle_arrow_scale_edit(self, keyval, delta):
+        print("handle_arrow_scale_edit")
+
+
+
+
 
 
 class RotatingEditCanvas(AbstractEditCanvas):
@@ -777,7 +1083,7 @@ class RotatingEditCanvas(AbstractEditCanvas):
         queue_draw()
         geometry_edit_finished()
         
-    Keyframes in form: [frame, [x, y, x_scale, y_scale, rotation] opacity]
+    Keyframes in form: [frame, [x, y, x_scale, y_scale, rotation], opacity, keyframe_type]
     """
     def __init__(self, editable_property, parent_editor):
         AbstractEditCanvas.__init__(self, editable_property, parent_editor)
@@ -789,7 +1095,9 @@ class RotatingEditCanvas(AbstractEditCanvas):
         self.y_scale = None
 
         self.draw_bounding_box = True # This may be set False at creation site.
-    
+
+        self.is_scale_locked = False  # This may be set True at creation site
+
     def create_edit_points_and_values(self):
         # creates untransformed edit shape to init array, values will be overridden shortly
         self.edit_points.append((self.source_width / 2, self.source_height / 2)) # center
@@ -863,6 +1171,31 @@ class RotatingEditCanvas(AbstractEditCanvas):
         self.keyframes.insert(active_kf_index, (frame, new_trans, opacity, kf_type))
         self._update_shape()
 
+    def clone_value_from_next(self, active_kf_index):
+        frame, trans, opacity, kf_type = self.keyframes.pop(active_kf_index)
+        
+        try:
+            frame_n, trans_n, opacity_n, kf_type_n = self.keyframes[active_kf_index]
+        except:
+            # No next keyframe
+            return
+    
+        self.keyframes.insert(active_kf_index, (frame, trans_n, opacity_n, kf_type))
+        self._update_shape()
+        self.parent_editor.update_slider_value_display(self.current_clip_frame)
+
+    def clone_value_from_prev(self, active_kf_index):
+        if active_kf_index == 0:
+            return
+            
+        frame, trans, opacity, kf_type = self.keyframes.pop(active_kf_index)
+        frame_n, trans_n, opacity_n, kf_type_n = self.keyframes[active_kf_index - 1]
+    
+        self.keyframes.insert(active_kf_index, (frame, trans_n, opacity_n, kf_type))
+        self._update_shape()
+        self.parent_editor.update_slider_value_display(self.current_clip_frame)
+        
+
     # -------------------------------------------------------- updating
     def _clip_frame_changed(self):
         self._update_shape()
@@ -886,19 +1219,13 @@ class RotatingEditCanvas(AbstractEditCanvas):
                     if kf_type == appconsts.KEYFRAME_DISCRETE:
                         self.set_geom(*rect)
                         return
-                    elif kf_type == appconsts.KEYFRAME_SMOOTH:
+                    else: # interpolated values
                         time_fract = float((self.current_clip_frame - frame)) / \
                                      float((frame_n - frame))
-                        frame_rect = self._get_interpolated_rect_smooth(time_fract, i, self.keyframes)
+                        frame_rect = self._get_interpolated_rect(time_fract, i)
                         self.set_geom(*frame_rect)
                         return
-                    else:
-                        time_fract = float((self.current_clip_frame - frame)) / \
-                                     float((frame_n - frame))
-                        frame_rect = self._get_interpolated_rect(rect, rect_n, time_fract)
-                        self.set_geom(*frame_rect)
-                        return
-            except: # past last frame, use its value  ( line: frame_n, rect_n, opacity_n = self.keyframes[i + 1] failed)
+            except Exception as e: # past last frame, use its value  ( line: frame_n, rect_n, opacity_n = self.keyframes[i + 1] failed)
                 self.set_geom(*rect)
                 return
 
@@ -910,53 +1237,31 @@ class RotatingEditCanvas(AbstractEditCanvas):
         self.rotation = rotation
         self._update_edit_points()
 
-    def _get_interpolated_rect(self, rect_1, rect_2, fract):
-        x1, y1, xs1, ys1, r1 = rect_1
-        x2, y2, xs2, ys2, r2 = rect_2
-        x = x1 + (x2 - x1) * fract
-        y = y1 + (y2 - y1) * fract
-        xs = xs1 + (xs2 - xs1) * fract
-        ys = ys1 + (ys2 - ys1) * fract
-        r = r1 + (r2 - r1) * fract
+    def _get_interpolated_rect(self, fract, i):
+        anim_value_x = self._create_anim_value(0)
+        x = anim_value_x.get_interpolated_value_internal_kf_type(i, fract)
+        anim_value_y = self._create_anim_value(1)
+        y = anim_value_y.get_interpolated_value_internal_kf_type(i, fract)
+        anim_value_x_scale = self._create_anim_value(2)
+        xs = anim_value_x_scale.get_interpolated_value_internal_kf_type(i, fract)
+        anim_value_y_scale = self._create_anim_value(3)
+        ys = anim_value_y_scale.get_interpolated_value_internal_kf_type(i, fract)
+        anim_value_rotation = self._create_anim_value(4)
+        r = anim_value_rotation.get_interpolated_value_internal_kf_type(i, fract)
+        
         return (x, y, xs, ys, r)
 
-    def _get_interpolated_rect_smooth(self, fract, i, keyframes):
-        prev = i
-        if i == 0:
-            prev_prev = 0
-        else:
-            prev_prev = i - 1
+    def _create_anim_value(self, value_index):
         
-        next = i + 1
-        if next >= len(keyframes):
-            next = len(keyframes) - 1
-        
-        next_next = next + 1
-        if next_next >= len(keyframes):
-            next_next = len(keyframes) - 1
+        value_keyframes = []
+    
+        for kf in self.keyframes:
+            frame, rect, opacity, kf_type = kf
+            value = rect[value_index] # x, y, x scale, y scale, rotation
+            value_keyframes.append((frame, value, kf_type))
 
-        frame, rect, opacity, kf_type = self.keyframes[prev_prev]
-        x0, y0, xs0, ys0, r0 = rect
+        return animatedvalue.AnimatedValue(value_keyframes)
 
-        frame, rect, opacity, kf_type = self.keyframes[prev]
-        x1, y1, xs1, ys1, r1 = rect
-
-        frame, rect, opacity, kf_type = self.keyframes[next]
-        x2, y2, xs2, ys2, r2 = rect
-
-        frame, rect, opacity, kf_type = self.keyframes[next_next]
-        x3, y3, xs3, ys3, r3 = rect
-
-        x = self.catmull_rom_interpolate(x0, x1, x2, x3, fract)
-        y = self.catmull_rom_interpolate(y0, y1, y2, y3, fract)
-        xs = self.catmull_rom_interpolate(xs0, xs1, xs2, xs3, fract)
-        ys = self.catmull_rom_interpolate(ys0, ys1, ys2, ys3, fract)
-        r = self.catmull_rom_interpolate(r0, r1, r2, r3, fract)
-
-        return (x, y, xs, ys, r)
-        
-
-        
     def handle_arrow_edit(self, keyval, delta):
         if keyval == Gdk.KEY_Left:
             self.shape_x -= delta
@@ -981,7 +1286,10 @@ class RotatingEditCanvas(AbstractEditCanvas):
             self.x_scale += delta
         
         self.y_scale = self.y_scale * (self.x_scale / old_scale)
-        
+
+        if self.is_scale_locked == True:
+            self.y_scale = self.x_scale 
+ 
     # --------------------------------------------------------- mouse events
     def _shape_press_event(self):
         self.start_edit_points = copy.deepcopy(self.edit_points)
@@ -1020,7 +1328,7 @@ class RotatingEditCanvas(AbstractEditCanvas):
             dist = viewgeom.distance(self.edit_points[POS_HANDLE], pp)
             orig_dist = viewgeom.distance(self.untrans_points[POS_HANDLE], self.untrans_points[X_SCALE_HANDLE])
             self.x_scale = dist / orig_dist
-            if CTRL_DOWN:
+            if CTRL_DOWN or self.is_scale_locked == True:
                 self.y_scale = self.x_scale
             self._update_edit_points()
         elif self.current_mouse_hit == Y_SCALE_HANDLE:
@@ -1029,7 +1337,7 @@ class RotatingEditCanvas(AbstractEditCanvas):
             dist = viewgeom.distance(self.edit_points[POS_HANDLE], pp)
             orig_dist = viewgeom.distance(self.untrans_points[POS_HANDLE], self.untrans_points[Y_SCALE_HANDLE])
             self.y_scale = dist / orig_dist
-            if CTRL_DOWN:
+            if CTRL_DOWN or self.is_scale_locked == True:
                 self.x_scale = self.y_scale
             self._update_edit_points()
         elif self.current_mouse_hit == ROTATION_HANDLE:
