@@ -2,7 +2,7 @@
     Flowblade Movie Editor is a nonlinear video editor.
     Copyright 2021 Janne Liljeblad.
 
-    This file is part of Flowblade Movie Editor <http://code.google.com/p/flowblade>.
+    This file is part of Flowblade Movie Editor <https://github.com/jliljebl/flowblade/>.
 
     Flowblade Movie Editor is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -20,7 +20,6 @@
 
 import gi
 gi.require_version('Gtk', '3.0')
-#gi.require_version('PangoCairo', '1.0')
 
 from gi.repository import GObject, GLib, Gio
 from gi.repository import Gtk, Gdk, GdkPixbuf
@@ -28,13 +27,11 @@ from gi.repository import Pango
 
 import datetime
 import glob
-import locale
 try:
     import mlt7 as mlt
 except:
     import mlt
 import os
-import pickle
 import shutil
 import subprocess
 import sys
@@ -51,20 +48,18 @@ import editorpersistance
 import fluxity
 import fluxityheadless
 import gui
-import guicomponents
 import guiutils
 import glassbuttons
+import gtkbuilder
 import gmicplayer
 import mediaplugin
 import mltinit
 import mltprofiles
-import mltfilters
 import positionbar
-import processutils
 import respaths
 import renderconsumer
 import toolsencoding
-import translations
+import toolguicomponents
 import threading
 import userfolders
 import utils
@@ -98,7 +93,7 @@ _mark_in = -1
 _mark_out = -1
 
 _current_profile_name = None
-_current_profile_index = None # We necesserily would not need this too.
+_current_profile_index = None # We necessarily would not need this too.
 
 _current_dimensions = None
 _current_fps = None
@@ -109,14 +104,10 @@ _encoding_panel = None
 
 _delay_timeout_id = -1
 
-builder = Gtk.Builder()
-
 _hamburger_popover = None
 
 #-------------------------------------------------- launch and inits
 def launch_scripttool(launch_data=None):
-    gui.save_current_colors()
-
     # no args yet
     args = ["profile_name:" + editorstate.PROJECT().profile.description()]
         
@@ -167,8 +158,6 @@ def main(root_path, force_launch=False):
 
     # Load editor prefs and apply themes images paths.
     editorpersistance.load()
-    if editorpersistance.prefs.theme != appconsts.LIGHT_THEME:
-        respaths.apply_dark_theme()
 
     # Init plugins module
     mediaplugin.init()
@@ -200,13 +189,11 @@ class ScriptToolApplication(Gtk.Application):
             MONITOR_HEIGHT = 400 # initial value, this gets changed when material is loaded
 
         # Themes
-        gui.apply_theme(editorpersistance.prefs.theme)
-        gui.load_current_colors()
+        gui.apply_theme()
 
         # Get launch profile and init player and display GUI params for it. 
         global _current_profile_name
         _current_profile_name = _get_arg_value(sys.argv, "profile_name")
-        _init_player_and_profile_data(_current_profile_name)
 
         # Show window.
         global _window
@@ -216,11 +203,32 @@ class ScriptToolApplication(Gtk.Application):
 
         os.putenv('SDL_WINDOWID', str(_window.monitor.get_window().get_xid()))
 
-        _init_playback()
-        update_length(_script_length)
+        # Set SDL consumer version to be used.
+        #if editorstate.mlt_version_is_greater_correct("7.28.0") or editorstate.force_sdl2 == True \
+        #    or editorstate.app_running_from == editorstate.RUNNING_FROM_FLATPAK:
+        if editorstate.force_sdl2 == True:
+            gmicplayer.set_sdl_consumer_version(gmicplayer.SDL_2)
+        else:
+            gmicplayer.set_sdl_consumer_version(gmicplayer.SDL_1)
         
+        # Launch player if we're using SDL 1 consumer.
+        if gmicplayer.get_sdl_consumer_version() == gmicplayer.SDL_1:
+            _init_player_and_profile_data(_current_profile_name)
+            _init_playback()
+            update_length(_script_length)
+        else:
+            # Launch SDL 2 player after data and gui exist if using that.
+            GLib.idle_add(_create_sdl_2_consumer)
+                        
         self.add_window(_window)
 
+# --------------------------------------- display
+def _create_sdl_2_consumer():
+
+    _init_player_and_profile_data(_current_profile_name)
+    _init_playback()
+    update_length(_script_length)
+    
 
 # ------------------------------------------------- folders init
 def _init_frames_dirs():
@@ -240,6 +248,8 @@ def _init_player_and_profile_data(profile_name):
     global _player, _ticker
     _ticker = utils.Ticker(_ticker_event, TICKER_DELAY)
     _player = gmicplayer.GmicPlayer(respaths.FLUXITY_EMPTY_BG_RES_PATH, _ticker)
+    _player.set_display_widget(_window.monitor)
+    _window.monitor.connect("size-allocate", lambda w, e:_player.display_resized())
 
 def _init_playback():
     _window.set_fps()
@@ -262,11 +272,12 @@ def _get_playback_tractor(length, range_frame_res_str=None, in_frame=-1, out_fra
     # Create tractor and tracks, in_frame
     tractor = mlt.Tractor()
     multitrack = tractor.multitrack()
-    track0 = mlt.Playlist()
+    profile = mltprofiles.get_profile(_current_profile_name)
+    track0 = mlt.Playlist(profile)
     multitrack.connect(track0, 0)
 
     bg_path = respaths.FLUXITY_EMPTY_BG_RES_PATH
-    profile = mltprofiles.get_profile(_current_profile_name)
+
     
     # If no producer provided, create producer displaying bg image with plugin icon.
     if range_frame_res_str == None:
@@ -404,18 +415,7 @@ def _hamburger_menu_callback(msg):
         _last_save_path = None
         _plugin_script_path = mediaplugin.get_plugin_script_path(msg)
         _reinit_init_playback()
-    
-def _get_menu_item(text, callback, data, sensitive=True):
-    item = Gtk.MenuItem.new_with_label(text)
-    item.connect("activate", callback, data)
-    item.show()
-    item.set_sensitive(sensitive)
-    return item
 
-def _add_separetor(menu):
-    sep = Gtk.SeparatorMenuItem()
-    sep.show()
-    menu.add(sep)
 
 #-------------------------------------------------- player buttons
 def prev_pressed(delta=-1):
@@ -535,7 +535,7 @@ def render_preview_frame():
     _mark_out = -1
     _window.update_marks_display()
     
-    fctx = fluxity.render_preview_frame(script, _last_save_path, frame, get_session_folder(), _profile_file_path)
+    fctx = fluxity.render_preview_frame(script, _last_save_path, frame, 200, get_session_folder(), _profile_file_path)
     _window.pos_bar.preview_range = None # frame or black for fail, no range anyway
         
     if fctx.error == None:
@@ -599,8 +599,8 @@ def _encode_settings_clicked():
     align = dialogutils.get_default_alignment(_encoding_panel)
     
     dialog = Gtk.Dialog(_("Video Encoding Settings"),
-                        _window,
-                        Gtk.DialogFlags.MODAL | Gtk.DialogFlags.DESTROY_WITH_PARENT,
+                        None,
+                        None,
                         (_("Cancel"), Gtk.ResponseType.REJECT,
                          _("Set Encoding"), Gtk.ResponseType.ACCEPT))
     dialog.vbox.pack_start(align, True, True, 0)
@@ -688,12 +688,9 @@ class ScriptToolWindow(Gtk.Window):
         # ---------------------------------------------------------------------- TOP ROW
         app_icon = GdkPixbuf.Pixbuf.new_from_file(respaths.IMAGE_PATH + "flowbladetoolicon.png")
         self.set_icon(app_icon)
-        hamburger_launcher_surface = guiutils.get_double_scaled_cairo_image("hamburger.png")
-        if editorpersistance.prefs.double_track_hights == False:
-            psize = 22
-        else:
-            psize = 44
-        self.hamburger_launcher = guicomponents.PressLaunch(self.hamburger_launch_pressed_popover, hamburger_launcher_surface, psize, psize)
+        hamburger_launcher_surface = guiutils.get_cairo_image("hamburger")
+        psize = 22
+        self.hamburger_launcher = toolguicomponents.PressLaunchSurface(self.hamburger_launch_pressed_popover, hamburger_launcher_surface, psize, psize)
         self.hamburger_launcher.widget.set_margin_bottom(7)
 
         self.reload_button = Gtk.Button(label=_("Reload Script"))
@@ -735,8 +732,6 @@ class ScriptToolWindow(Gtk.Window):
         # -------------------------------------------------------------------- RIGHT SIDE: MONITOR, CONTROLS, ENCODING 
         black_box = Gtk.EventBox()
         black_box.add(Gtk.Label())
-        bg_color = Gdk.Color(red=0.0, green=0.0, blue=0.0)
-        black_box.modify_bg(Gtk.StateType.NORMAL, bg_color)
         self.monitor = black_box  # This could be any GTK+ widget (that is not "windowless"), only its XWindow draw rect 
                                   # is used to position and scale SDL overlay that actually displays video.
         self.monitor.set_size_request(MONITOR_WIDTH, MONITOR_HEIGHT)
@@ -749,7 +744,7 @@ class ScriptToolWindow(Gtk.Window):
         self.monitors_switcher.set_visible_child_name(CAIRO_DRAW_MONITOR)
 
         # Control row
-        self.tc_display = guicomponents.MonitorTCDisplay(56)
+        self.tc_display = toolguicomponents.MonitorTCDisplay(56)
         self.tc_display.use_internal_frame = True
         self.tc_display.widget.set_valign(Gtk.Align.CENTER)
         self.tc_display.use_internal_fps = True
@@ -759,13 +754,12 @@ class ScriptToolWindow(Gtk.Window):
         self.pos_bar.set_listener(self.position_listener)
         self.pos_bar.mouse_press_listener = self.pos_bar_press_listener
         
-        pos_bar_frame = Gtk.Frame()
+        pos_bar_frame = Gtk.HBox()
         pos_bar_frame.add(self.pos_bar.widget)
-        pos_bar_frame.set_shadow_type(Gtk.ShadowType.ETCHED_IN)
         pos_bar_frame.set_margin_top(10)
         pos_bar_frame.set_margin_bottom(9)
-        pos_bar_frame.set_margin_left(6)
-        pos_bar_frame.set_margin_right(2)
+        pos_bar_frame.set_margin_start(6)
+        pos_bar_frame.set_margin_end(2)
                                                   
         self.control_buttons = glassbuttons.PlayerButtons()
         if editorpersistance.prefs.play_pause == False:
@@ -864,9 +858,9 @@ class ScriptToolWindow(Gtk.Window):
         marks_row.pack_start(out_row, True, True, 0)
         marks_row.pack_start(length_row, True, True, 0)
 
-        self.out_folder = Gtk.FileChooserButton(_("Select Folder"))
+        self.out_folder = gtkbuilder.get_file_chooser_button(_("Select Folder"))
         self.out_folder.set_action(Gtk.FileChooserAction.SELECT_FOLDER)
-        self.out_folder.connect("selection-changed", self.folder_selection_changed) 
+        self.out_folder.connect_selection_changed(None, self.folder_selection_changed)
         self.out_label = Gtk.Label(label=_("Frames Folder:"))
 
         self.frame_name = Gtk.Entry()
@@ -881,7 +875,7 @@ class ScriptToolWindow(Gtk.Window):
         self.encode_check_label = Gtk.Label(_("Encode Video"))
         self.encode_check = Gtk.CheckButton()
         self.encode_check.set_active(False)
-        self.encode_check.set_margin_right(4)
+        self.encode_check.set_margin_end(4)
         self.encode_check.connect("toggled", lambda w:self.update_encode_sensitive())
 
         self.encode_settings_button = Gtk.Button(label=_("Encoding settings"))
@@ -953,7 +947,7 @@ class ScriptToolWindow(Gtk.Window):
         preview_render_vbox.pack_start(preview_panel, True, True, 0)
         preview_render_vbox.pack_start(render_vbox, False, False, 0)
         preview_render_vbox.pack_start(editor_buttons_row, False, False, 0)
-        preview_render_vbox.set_margin_left(4)
+        preview_render_vbox.set_margin_start(4)
 
         # --------------------------------------------------- LEFT SIDE: SCRIPTING
         # --------------------------------------------------- LEFT SIDE: SCRIPTING
@@ -990,7 +984,7 @@ class ScriptToolWindow(Gtk.Window):
         script_vbox = Gtk.Paned.new(Gtk.Orientation.VERTICAL) 
         script_vbox.pack1(script_sw, True, False)
         script_vbox.pack2(out_sw, False, False)
-        script_vbox.set_margin_right(4)
+        script_vbox.set_margin_end(4)
         
 
         # ------------------------------------------------------------ BUILD WINDOW
@@ -1074,7 +1068,7 @@ class ScriptToolWindow(Gtk.Window):
             self.render_status_info.set_markup("<small>" + info_str + "</small>")
             self.render_button.set_sensitive(True)
             
-    def folder_selection_changed(self, chooser):
+    def folder_selection_changed(self, chooser, data):
         self.update_render_status_info()
 
     def hamburger_launch_pressed_popover(self, widget, event):
@@ -1120,28 +1114,6 @@ class ScriptToolWindow(Gtk.Window):
         action.connect("activate", lambda w, e, msg:_hamburger_menu_callback(msg), msg_str)
         _app.add_action(action)
 
-    # This stuff is some testing.
-    """
-    def add_menu_action_check(self, menu, label, item_id, msg_str):
-        action = Gio.SimpleAction.new_stateful(name=item_id, parameter_type=None, state=GLib.Variant.new_boolean(True))
-        action.connect("activate", lambda w, e, msg:_hamburger_menu_callback(msg), msg_str)
-        _app.add_action(action)
-
-        menu_item = Gio.MenuItem.new(label, "app." + item_id)
-        menu_item.set_action_and_target_value( "app." + item_id, None)
-        menu.append_item(menu_item)
-
-    def add_menu_action_radio(self, menu, label, item_id, msg_str):
-        target_variat = GLib.Variant.new_string("stringiii")
-        action = Gio.SimpleAction.new_stateful(name=item_id, parameter_type= GLib.VariantType.new("s"), state=target_variat)
-        action.connect("activate", lambda w, e, msg:_hamburger_menu_callback(msg), msg_str)
-        _app.add_action(action)
-
-        menu_item = Gio.MenuItem.new(label, "app." + item_id)
-        menu_item.set_action_and_target_value( "app." + item_id, target_variat)
-        menu.append_item(menu_item)
-    """
-    
     def set_active_state(self, active):
         self.monitor.set_sensitive(active)
         self.pos_bar.widget.set_sensitive(active)
@@ -1319,29 +1291,34 @@ class FluxityRangeRenderer(threading.Thread):
     def run(self):
         GLib.idle_add(_update_buffer_text, self.rendering_txt)
 
-        # GUI quarantees valid range here.
+        # GUI guarantees valid range here.
         in_frame = _mark_in
         self.in_frame = in_frame
         out_frame = _mark_out
         self.out_frame = out_frame  
-
-        ccrutils.init_session_folders(SCRIPT_TOOL_SESSION_ID)
+        
+        # TODO: Make user settable
+        generator_length = 200
+            
+        parent_folder = userfolders.get_temp_render_dir()
+    
+        ccrutils.init_session_folders(parent_folder, SCRIPT_TOOL_SESSION_ID)
         
         # Create temp script file from text area contents.
-        tmp_script_file = ccrutils.get_session_folder(SCRIPT_TOOL_SESSION_ID) + "/temp_script"
+        tmp_script_file = ccrutils.get_session_folder(parent_folder, SCRIPT_TOOL_SESSION_ID) + "/temp_script"
         with atomicfile.AtomicFileWriter(tmp_script_file, "w") as afw:
             write_file = afw.get_file()
             f = afw.get_file()
             f.write(self.script)
             
-        frames_folder = ccrutils.get_render_folder_for_session_id(SCRIPT_TOOL_SESSION_ID)
+        frames_folder = ccrutils.get_render_folder_for_session_id(parent_folder, SCRIPT_TOOL_SESSION_ID)
         
         err_msg, edit_data = fluxity.get_script_default_edit_data(self.script, tmp_script_file, frames_folder, self.profile_file_path)
         if err_msg != None:
             GLib.idle_add(_show_error, err_msg)
             return
 
-        _launch_headless_render(SCRIPT_TOOL_SESSION_ID, tmp_script_file, edit_data, frames_folder, in_frame, out_frame + 1)
+        _launch_headless_render(SCRIPT_TOOL_SESSION_ID, tmp_script_file, edit_data, frames_folder, generator_length, in_frame, out_frame + 1)
 
         GLib.timeout_add(200, _preview_render_update, self)
 
@@ -1379,14 +1356,16 @@ def _preview_render_update(render_thread):
 
 def _preview_range_render_complete(render_thread):
         # Get error and log messages.
-        proc_fctx_dict = ccrutils.read_range_render_data(SCRIPT_TOOL_SESSION_ID)
+        parent_folder = userfolders.get_temp_render_dir()
+        
+        proc_fctx_dict = ccrutils.read_range_render_data(parent_folder, SCRIPT_TOOL_SESSION_ID)
         error_msg, log_msg = _get_range_render_messages(proc_fctx_dict)
 
         if error_msg == None:
             frame_file = proc_fctx_dict["0"] # First written file saved here.
             in_frame, out_frame = render_thread.in_frame, render_thread.out_frame
             resource_name_str = utils.get_img_seq_resource_name(frame_file)
-            frames_folder = ccrutils.get_render_folder_for_session_id(SCRIPT_TOOL_SESSION_ID)
+            frames_folder = ccrutils.get_render_folder_for_session_id(parent_folder, SCRIPT_TOOL_SESSION_ID)
             range_resourse_mlt_path = frames_folder + "/" + resource_name_str
             
             new_playback_producer = _get_playback_tractor(_script_length, range_resourse_mlt_path, in_frame, out_frame)
@@ -1450,6 +1429,9 @@ class FluxityPluginRenderer(threading.Thread):
         self.in_frame = in_frame
         self.out_frame = out_frame
         
+        # TODO: Make user settable
+        generator_length = 200
+        
         # not used !!!
         frame_name = _window.frame_name.get_text()
 
@@ -1462,16 +1444,17 @@ class FluxityPluginRenderer(threading.Thread):
             file_path = os.path.join(out_folder, frame_file)
             os.remove(file_path)
 
-        ccrutils.init_session_folders(SCRIPT_TOOL_SESSION_ID)
+        parent_folder = userfolders.get_temp_render_dir()
+        ccrutils.init_session_folders(parent_folder, SCRIPT_TOOL_SESSION_ID)
         
         # Create temp script file from text area contents.
-        tmp_script_file = ccrutils.get_session_folder(SCRIPT_TOOL_SESSION_ID) + "/temp_script"
+        tmp_script_file = ccrutils.get_session_folder(parent_folder, SCRIPT_TOOL_SESSION_ID) + "/temp_script"
         with atomicfile.AtomicFileWriter(tmp_script_file, "w") as afw:
             write_file = afw.get_file()
             f = afw.get_file()
             f.write(script_text)
             
-        frames_folder = ccrutils.get_render_folder_for_session_id(SCRIPT_TOOL_SESSION_ID)
+        frames_folder = ccrutils.get_render_folder_for_session_id(parent_folder, SCRIPT_TOOL_SESSION_ID)
         
         err_msg, edit_data = fluxity.get_script_default_edit_data(script_text, tmp_script_file, frames_folder, profile_file_path)
         if err_msg != None:
@@ -1479,7 +1462,7 @@ class FluxityPluginRenderer(threading.Thread):
             return
 
         self.frames_render_in_prgress = True
-        _launch_headless_render(SCRIPT_TOOL_SESSION_ID, tmp_script_file, edit_data, frames_folder, in_frame, out_frame + 1)
+        _launch_headless_render(SCRIPT_TOOL_SESSION_ID, tmp_script_file, edit_data, frames_folder, generator_length, in_frame, out_frame + 1)
         
         self.render_start_time = datetime.datetime.now()
         GLib.timeout_add(150, _output_render_update, self)
@@ -1491,7 +1474,9 @@ class FluxityPluginRenderer(threading.Thread):
         if self.abort == True:
             return
         
-        proc_fctx_dict = ccrutils.read_range_render_data(SCRIPT_TOOL_SESSION_ID)
+        parent_folder = userfolders.get_temp_render_dir()
+        
+        proc_fctx_dict = ccrutils.read_range_render_data(parent_folder, SCRIPT_TOOL_SESSION_ID)
         error_msg, log_msg = _get_range_render_messages(proc_fctx_dict)
 
         # Set GUI state and exit on error.
@@ -1549,15 +1534,16 @@ class FluxityPluginRenderer(threading.Thread):
     def abort_render(self):
         self.abort = True
         self.frames_render_in_prgress = False
-        ccrutils.abort_render(SCRIPT_TOOL_SESSION_ID)
+        parent_folder = userfolders.get_temp_render_dir()
+        ccrutils.abort_render(parent_folder, SCRIPT_TOOL_SESSION_ID)
 
         self.shutdown()
                          
         _window.render_percentage.set_markup("<small>" + _("Render stopped!") + "</small>")
         self.set_render_stopped_gui_state()
 
-    def show_progress(self, persentage, fraction, set_stopped):
-        _window.render_percentage.set_markup(persentage)
+    def show_progress(self, percentage, fraction, set_stopped):
+        _window.render_percentage.set_markup(percentage)
         _window.render_progress_bar.set_fraction(fraction)
         if set_stopped:
             self.set_render_stopped_gui_state()
@@ -1622,10 +1608,12 @@ def _output_render_update(render_thread):
         _window.render_progress_bar.set_fraction(progress)
         return True # Function will be called again.
 
-def _launch_headless_render(session_id, script_path, edit_data, frames_folder, range_in, range_out):
+def _launch_headless_render(session_id, script_path, edit_data, frames_folder, generator_length, range_in, range_out):
 
-    ccrutils.init_session_folders(session_id)
-    fluxityheadless.clear_flag_files(session_id)
+    parent_folder = userfolders.get_temp_render_dir()
+    
+    ccrutils.init_session_folders(parent_folder, session_id)
+    fluxityheadless.clear_flag_files(parent_folder, session_id)
 
     # Delete old rendered frames.
     old_files = glob.glob(frames_folder + "/*")
@@ -1639,12 +1627,14 @@ def _launch_headless_render(session_id, script_path, edit_data, frames_folder, r
 
     render_data.do_video_render = False 
 
-    fluxityheadless.set_render_data(session_id, render_data)
+    fluxityheadless.set_render_data(parent_folder, session_id, render_data)
 
-    ccrutils.write_misc_session_data(session_id, "fluxity_plugin_edit_data", edit_data)
+    ccrutils.write_misc_session_data(parent_folder, session_id, "fluxity_plugin_edit_data", edit_data)
     
-    args = ("session_id:" + session_id, 
+    args = ("session_id:" + session_id,
+            "parent_folder:" + parent_folder,
             "script:" + script_path,
+            "generator_length:" + str(generator_length),
             "range_in:" + str(range_in),
             "range_out:"+ str(range_out),
             "profile_desc:" + _current_profile_name.replace(" ", "_"))  # Here we have our own string space handling, maybe change later..
@@ -1659,10 +1649,12 @@ def _launch_headless_render(session_id, script_path, edit_data, frames_folder, r
 
 def _get_render_status(session_id, render_range_in, render_range_out):
     # Returns tuple (completed, step, progress)
-    if fluxityheadless.session_render_complete(session_id) == True:
+    parent_folder = userfolders.get_temp_render_dir()
+    
+    if fluxityheadless.session_render_complete(parent_folder, session_id) == True:
         return (True, None, None, None)
     else:
-        status = fluxityheadless.get_session_status(session_id)
+        status = fluxityheadless.get_session_status(parent_folder, session_id)
         if status != None:
             step, frame, length, elapsed = status
             frame = int(frame)
